#include "WindowManager.h"

WindowManager WindowManager::instance=WindowManager();

WindowManager::WindowManager() : isFullScreen(FULLSCREEN), event(0), window(0), windowTmp(0), open(false) {
}

WindowManager::~WindowManager() {
    delete event;
}

WindowManager* WindowManager::getInstance() {
    return &instance;
}

int WindowManager::createWindow(string title, string iconName) {
    if (window == 0) {
        if(SDL_Init(SDL_INIT_VIDEO) == -1) {
            printf("Could not load SDL : %s\n", SDL_GetError());
            return -1;
        }
        atexit(SDL_Quit);

        SDL_WM_SetCaption(title.c_str(), NULL);
        SDL_Surface* icon = SDL_LoadBMP(iconName.c_str());
        SDL_SetColorKey(icon, SDL_SRCCOLORKEY, SDL_MapRGB(icon->format,0,0,0));
        SDL_WM_SetIcon(icon, NULL);

        SDL_ShowCursor(isFullScreen ? SDL_DISABLE : SDL_ENABLE);

		// Xbox cut resolution in half to improve speed
        window = SDL_SetVideoMode(320, 240, 32, SDL_HWSURFACE|SDL_DOUBLEBUF|SDL_FULLSCREEN);
        windowTmp = SDL_CreateRGBSurface(SDL_HWSURFACE, 640, 480, 32, 0, 0, 0, 0);

        //if (isFullScreen) SDL_Delay(2000); // wait the window is displayed

		//Init for Xbox joy
		if ( SDL_InitSubSystem( SDL_INIT_JOYSTICK ) < 0 )
		{	
			XLaunchNewImage(NULL, NULL);
		}
		
        event = new Event();
		event->JOYPAD = SDL_JoystickOpen(0);
		SDL_JoystickEventState(SDL_ENABLE);

        lastAnimTime = SDL_GetTicks();
        open = true;
    }
    return 0;
}

bool WindowManager::isOpened() {
    return open;
}

Event* WindowManager::getEvent() {

	//Xbox Dpad and Joy Sticks
	XPAD   = SDL_JoystickGetHat (event->JOYPAD, 0);
	XAXIS  = SDL_JoystickGetAxis(event->JOYPAD, 0);
	YAXIS  = SDL_JoystickGetAxis(event->JOYPAD, 1);
	LXAXIS = SDL_JoystickGetAxis(event->JOYPAD, 2);
	LYAXIS = SDL_JoystickGetAxis(event->JOYPAD, 3);

	SDL_Event sdlEvent;
	Uint8* keys = SDL_GetKeyState(NULL);
	// todo switch case
    while(SDL_PollEvent(&sdlEvent))
	{
		if(sdlEvent.type == SDL_JOYBUTTONDOWN)
		{
			//Xbox buttons
			XBUTTON = sdlEvent.jbutton.button;

			if(XBUTTON == JOY_BTTN_START)
				keys[SDLK_RETURN] = SDL_PRESSED;

			if(XBUTTON == JOY_BTTN_X)
				keys[SDLK_x] = SDL_PRESSED;

			if(XBUTTON == JOY_BTTN_B)
			{
				keys[SDLK_c] = SDL_PRESSED;
				keys[SDLK_SPACE] = SDL_PRESSED;
				keys[SDLK_LSHIFT] = SDL_PRESSED;
			}
				
			if(XBUTTON == JOY_BTTN_A)
				keys[SDLK_z] = SDL_PRESSED;

			if(XBUTTON == JOY_BTTN_Y)
			{
				keys[SDLK_p] = SDL_PRESSED;
				keys[SDLK_v] = SDL_PRESSED;
			}

			if(XBUTTON == JOY_BTTN_RTRIG)
				keys[SDLK_RCTRL] = SDL_PRESSED;

			if(XBUTTON == JOY_BTTN_LTRIG)
				keys[SDLK_o]  = SDL_PRESSED;
				
			if(XBUTTON == JOY_BTTN_WHITE)
				keys[SDLK_i]    = SDL_PRESSED;
				
			if(XBUTTON == JOY_BTTN_BLACK)
				keys[SDLK_F1] = SDL_PRESSED;

			if(XBUTTON == JOY_BTTN_BACK)
				keys[SDLK_ESCAPE] = SDL_PRESSED;

			if(XBUTTON == JOY_BTTN_RSTICK)
				keys[SDLK_t] = SDL_PRESSED;

			if(XBUTTON == JOY_BTTN_LSTICK)
				resize_screen();
		}

		if(sdlEvent.type == SDL_JOYBUTTONUP)
		{
			//Xbox buttons
			XBUTTON = sdlEvent.jbutton.button;

			if(XBUTTON == JOY_BTTN_START)
				keys[SDLK_RETURN] = SDL_RELEASED;

			if(XBUTTON == JOY_BTTN_X)
				keys[SDLK_x] = SDL_RELEASED;

			if(XBUTTON == JOY_BTTN_B)
			{
				keys[SDLK_c] = SDL_RELEASED;
				keys[SDLK_SPACE] = SDL_RELEASED;
				keys[SDLK_LSHIFT] = SDL_RELEASED;
			}
				
			if(XBUTTON == JOY_BTTN_A)
				keys[SDLK_z] = SDL_RELEASED;

			if(XBUTTON == JOY_BTTN_Y)
			{
				keys[SDLK_p] = SDL_RELEASED;
				keys[SDLK_v] = SDL_RELEASED;
			}

			if(XBUTTON == JOY_BTTN_RTRIG)
				keys[SDLK_RCTRL] = SDL_RELEASED;

			if(XBUTTON == JOY_BTTN_LTRIG)
				keys[SDLK_o]  = SDL_RELEASED;
				
			if(XBUTTON == JOY_BTTN_WHITE)
				keys[SDLK_i]    = SDL_RELEASED;
				
			if(XBUTTON == JOY_BTTN_BLACK)
				keys[SDLK_F1] = SDL_RELEASED;

			if(XBUTTON == JOY_BTTN_BACK)
				keys[SDLK_ESCAPE] = SDL_RELEASED;

			if(XBUTTON == JOY_BTTN_RSTICK)
				keys[SDLK_t] = SDL_RELEASED;	
		}
	}

	if(XPAD == SDL_HAT_UP || YAXIS < -JOY_DEADZONE)
		keys[SDLK_UP]     = SDL_PRESSED;
	else
		keys[SDLK_UP]     = SDL_RELEASED;

	if(XPAD == SDL_HAT_DOWN || YAXIS > JOY_DEADZONE)
		keys[SDLK_DOWN]   = SDL_PRESSED;
	else
		keys[SDLK_DOWN]   = SDL_RELEASED;

	if(XPAD == SDL_HAT_LEFT || XAXIS < -JOY_DEADZONE)
		keys[SDLK_LEFT]   = SDL_PRESSED;
	else
		keys[SDLK_LEFT]   = SDL_RELEASED;

	if(XPAD == SDL_HAT_RIGHT || XAXIS > JOY_DEADZONE)
		keys[SDLK_RIGHT]  = SDL_PRESSED;
	else
		keys[SDLK_RIGHT]   = SDL_RELEASED;

	if(XPAD == SDL_HAT_RIGHTUP)
	{
		keys[SDLK_UP]	  = SDL_PRESSED;
		keys[SDLK_RIGHT]  = SDL_PRESSED;
	}

	if(XPAD == SDL_HAT_RIGHTDOWN)
	{
		keys[SDLK_DOWN]   = SDL_PRESSED;
		keys[SDLK_RIGHT]  = SDL_PRESSED;
	}

	if(XPAD == SDL_HAT_LEFTUP)
	{
		keys[SDLK_UP]    = SDL_PRESSED;
		keys[SDLK_LEFT]  = SDL_PRESSED;
	}


	if(XPAD == SDL_HAT_LEFTDOWN)
	{
		keys[SDLK_DOWN] = SDL_PRESSED;
		keys[SDLK_LEFT] = SDL_PRESSED;
	}

    event->update(keys);

    if (event->isPushed(RESIZE)) {
        isFullScreen = ! isFullScreen;

        isFullScreen ? SDL_ShowCursor(SDL_DISABLE) : SDL_ShowCursor(SDL_ENABLE);
        window = SDL_SetVideoMode(320, 240, 32, isFullScreen ? SDL_HWSURFACE|SDL_DOUBLEBUF|SDL_FULLSCREEN : SDL_HWSURFACE|SDL_DOUBLEBUF);

    }

    while (SDL_PollEvent(&sdlEvent)) {
        switch (sdlEvent.type) {
            case SDL_QUIT : event->setEvent(QUIT, true); break;
        }
    }

    return event;
}

void WindowManager::close() {
    open = false;
}

void WindowManager::display() {

    //SDL_Surface* tmp = zoomSurface(windowTmp, 2, 2, 0);
    SDL_BlitSurface(windowTmp, NULL, window, NULL);
    //SDL_FreeSurface(tmp);

    SDL_Flip(window);


	// Frame rate more consistant if we don't delay here
    if (SDL_GetTicks() < lastAnimTime + (1000 / 60)) 
		SDL_Delay(1);

    lastAnimTime = SDL_GetTicks();
}

void WindowManager::draw(SDL_Surface* object) {
    SDL_BlitSurface(object, NULL, windowTmp, NULL);
}

void WindowManager::draw(WImage* image, int srcX, int srcY, int srcW, int srcH, int dstX, int dstY, int alpha) {
    SDL_Rect src;
    src.x = srcX;
    src.y = srcY;
    src.w = srcW;
    src.h = srcH;

    SDL_Rect dst;
    dst.x = dstX;
    dst.y = dstY;

    SDL_BlitSurface(image->getImage(), &src, windowTmp, &dst);
}

void WindowManager::exit() {
    SDL_FreeSurface(windowTmp);

    SDL_ShowCursor(SDL_ENABLE);
    SDL_Quit();
	XLaunchNewImage(NULL, NULL);
}
